﻿using Ozeki.Network;
using System;
using System.Xml.Linq;
using Ozeki.Media;
using Ozeki.VoIP;

namespace _07_IVR_database
{
    class Program
    {
        public static Menu Menu = new Menu();

        static void Main(string[] args)
        {
            ShowHelp();
            
            Softphone softphone = new Softphone();

            SipAccountInitialization(softphone);
            DatabaseAccountInitialization();

            softphone.IncomigCall += softphone_IncomigCall;

            Console.ReadLine();
        }

        private static void ShowHelp()
        {
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine("This is an example code for Interactive Voice Response (IVR) written in C#. This is a technology that allows a computer to interact with humans through the use of voice and DTMF tones input via keypad.");
            Console.WriteLine("We are going to create the IVR system by using an XML code. This makes the creation of a complex IVR so much easier.");
            Console.WriteLine("You will need to  setup your SIP account (your authentication ID, username, display name etc.). If the registration succeeded and you call the number of the IVR you will hear a greeting message.");
            Console.WriteLine("After this you can choose between two menu options.\n");

            Console.ForegroundColor = ConsoleColor.White;

        }

        static void softphone_IncomigCall(object sender, VoIPEventArgs<Ozeki.VoIP.IPhoneCall> e)
        {
            var menu = ReadXML();

            if (menu != null)
                menu.Start(e.Item);
            else
                e.Item.Reject();
        }

        private static Menu ReadXML()
        {
            DBConnect database = new DBConnect();
            string ivrXml = @"" + database.Select();

            var menu = new Menu();
            MenuLevel(ivrXml, menu);
            return menu;
        }

        private static void MenuLevel(string ivrXml, Menu menu)
        {
          try
          {
            var xelement = XElement.Parse(ivrXml);

            var menuElement = xelement.Element("menu");

            if (menuElement == null)
            {
                Console.WriteLine("Wrong XML code!");
                return;
            }

            var menuInit = menuElement.Element("init");

            foreach (var initElements in menuInit.Elements())
            {
                switch (initElements.Name.ToString())
                {
                    case "play":
                        menu.AddInitCommand(new PlayCommand(initElements.Value));
                        break;
                    case "speak":
                        menu.AddInitCommand(new SpeakCommand(initElements.Value));
                        break;
                }
            }

            var menuKeys = menuElement.Element("keys");

            foreach (var key in menuKeys.Elements("key"))
            {
                var pressedKeyAttribute = key.Attribute("pressed");

                if (pressedKeyAttribute == null)
                {
                    Console.WriteLine("Invalid ivr xml, keypress has no value!");
                    return;
                }

                int pressedKey;

                if (!Int32.TryParse(pressedKeyAttribute.Value, out pressedKey))
                {
                    Console.WriteLine("You did not add any number!");
                }

                foreach (var element in key.Elements())
                {
                    switch (element.Name.ToString())
                    {
                        case "play":
                            menu.AddKeypressCommand(pressedKey, new PlayCommand(element.Value));
                            break;
                        case "speak":
                            menu.AddKeypressCommand(pressedKey, new SpeakCommand(element.Value));
                            break;
                        case "menu":
                            Menu innerMenu = new Menu();
                            menu.AddKeypressCommand(pressedKey, innerMenu);
                            MenuLevel(key.ToString(), innerMenu);
                            break;
                        default:
                            return;
                    }
                }

            }
          }
          catch (Exception ex)
          {
              Console.WriteLine(ex.Message);
              Console.WriteLine("Invalid ivr xml");
          }
        }

        private static void SipAccountInitialization(Softphone softphone)
        {
            Console.WriteLine("Please setup your SIP account!\n");
            Console.WriteLine("Please set your authentication ID: ");
            var authenticationId = Read("authenticationId", true);

            Console.WriteLine("Please set your user name (default:" + authenticationId + "): ");
            var userName = Read("userName", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            Console.WriteLine("Please set your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("displayName", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            Console.WriteLine("Please set your registration password: ");
            var registrationPassword = Read("registrationPassword", true);

            Console.WriteLine("Please set the domain name (default: your local host): ");
            var domainHost = Read("domainHost", false);
            if (string.IsNullOrEmpty(domainHost))
                domainHost = NetworkAddressHelper.GetLocalIP().ToString();

            Console.WriteLine("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("domainPort", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register...\n");
            softphone.Register(true, displayName, userName, authenticationId, registrationPassword, domainHost, domainPort);
        }

        private static void DatabaseAccountInitialization()
        {
            Console.WriteLine("Please set your database user name (default: root): ");
            User = Read("user", false);
            if (string.IsNullOrEmpty(User))
                User = "root";

            Console.WriteLine("Please set your database password (default is nothing): ");
            Password = Read("password", false);
            if (string.IsNullOrEmpty(Password))
                Password = "";

            Console.WriteLine("Please set your database host (default: localhost): ");
            Server = Read("server", false);
            if (string.IsNullOrEmpty(Server))
                Server = "localhost";

            Console.WriteLine("Please set your database name (default: ivrdata): ");
            DatabName = Read("databName", false);
            if (string.IsNullOrEmpty(DatabName))
                DatabName = "ivrdata";

            Console.WriteLine("Please set your table name (default: ivr): ");
            Table = Read("table", false);
            if (string.IsNullOrEmpty(Table))
                Table = "ivr";
        }

        public static string Server { get; private set; }
        public static string User { get; private set; }
        public static string Password { get; private set; }
        public static string DatabName { get; private set; }
        public static string Table { get; private set; }

        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                {
                    return input;
                }

                if (!string.IsNullOrEmpty(input))
                {
                    return input;
                }

                Console.WriteLine(inputName + " cannot be empty!");
                Console.WriteLine(inputName + ": ");
            }
        }
    }
}
